package com.xxh.cloud.healthy.behavior.framework.modules.client.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.xxh.cloud.healthy.behavior.client.api.PatientBehaviorClientService;
import com.xxh.cloud.healthy.behavior.client.api.bean.bo.PatientBehaviorSecureBO;
import com.xxh.cloud.healthy.behavior.client.api.bean.dto.PatientBehaviorSaveClientDTO;
import com.xxh.cloud.healthy.behavior.client.api.bean.dto.PatientBehaviorUploadClientDTO;
import com.xxh.cloud.healthy.behavior.client.api.util.BehaviorSecureUtil;
import com.xxh.cloud.healthy.behavior.framework.modules.bean.entity.PatientBehaviorInfo;
import com.xxh.cloud.healthy.behavior.framework.modules.bean.service.PatientBehaviorInfoService;
import com.xxh.cloud.healthy.behavior.framework.modules.biz.bean.dto.PatientBehaviorAddDTO;
import com.xxh.cloud.healthy.behavior.framework.modules.biz.helper.PatientBehaviorHelper;
import com.xxh.cloud.healthy.behavior.framework.modules.biz.service.PatientBehaviorBizService;

import cn.hutool.core.util.StrUtil;

@Service
public class PatientBehaviorClientServiceImpl implements PatientBehaviorClientService {

    @Autowired
    private PatientBehaviorBizService patientBehaviorBizService;
    @Autowired
    private PatientBehaviorInfoService patientBehaviorInfoService;

    @Override
    public void add(PatientBehaviorUploadClientDTO params) {
        if (StrUtil.isBlank(params.getSecureData())) {
            return;
        }
        // 解析加密数据
        PatientBehaviorSecureBO patientBehaviorSecureBO = BehaviorSecureUtil.decryptData(params.getSecureData());

        PatientBehaviorAddDTO entity = new PatientBehaviorAddDTO();
        entity.setPatientId(patientBehaviorSecureBO.getPatientId());
        entity.setUserId(patientBehaviorSecureBO.getUserId());
        entity.setContentType(patientBehaviorSecureBO.getContentType());
        entity.setContentId(patientBehaviorSecureBO.getContentId());
        entity.setSourceOneType(patientBehaviorSecureBO.getSourceOneType());
        entity.setSourceOneId(patientBehaviorSecureBO.getSourceOneId());
        entity.setSourceTwoType(patientBehaviorSecureBO.getSourceTwoType());
        entity.setSourceTwoId(patientBehaviorSecureBO.getSourceTwoId());
        entity.setSourceThreeType(patientBehaviorSecureBO.getSourceThreeType());
        entity.setSourceThreeId(patientBehaviorSecureBO.getSourceThreeId());
        entity.setSourceFourType(patientBehaviorSecureBO.getSourceFourType());
        entity.setSourceFourId(patientBehaviorSecureBO.getSourceFourId());
        entity.setSourceFiveType(patientBehaviorSecureBO.getSourceFiveType());
        entity.setSourceFiveId(patientBehaviorSecureBO.getSourceFiveId());
        entity.setOperatedType(params.getOperatedType());
        entity.setOperatedDuration(params.getOperatedDuration());
        entity.setIp(params.getIp());
        entity.setUa(params.getUa());

        entity.setOperatedContent(params.getOperatedContent());

        patientBehaviorBizService.add(entity);
    }

    @Override
    public void addBatch(List<PatientBehaviorUploadClientDTO> params) {
        List<PatientBehaviorAddDTO> list = new ArrayList<>();
        params.forEach(i -> {

            // 解析加密数据
            PatientBehaviorSecureBO patientBehaviorSecureBO = BehaviorSecureUtil.decryptData(i.getSecureData());

            PatientBehaviorAddDTO addDTO = new PatientBehaviorAddDTO();
            addDTO.setPatientId(patientBehaviorSecureBO.getPatientId());
            addDTO.setUserId(patientBehaviorSecureBO.getUserId());
            addDTO.setContentType(patientBehaviorSecureBO.getContentType());
            addDTO.setContentId(patientBehaviorSecureBO.getContentId());
            addDTO.setSourceOneType(patientBehaviorSecureBO.getSourceOneType());
            addDTO.setSourceOneId(patientBehaviorSecureBO.getSourceOneId());
            addDTO.setSourceTwoType(patientBehaviorSecureBO.getSourceTwoType());
            addDTO.setSourceTwoId(patientBehaviorSecureBO.getSourceTwoId());
            addDTO.setSourceThreeType(patientBehaviorSecureBO.getSourceThreeType());
            addDTO.setSourceThreeId(patientBehaviorSecureBO.getSourceThreeId());
            addDTO.setSourceFourType(patientBehaviorSecureBO.getSourceFourType());
            addDTO.setSourceFourId(patientBehaviorSecureBO.getSourceFourId());
            addDTO.setSourceFiveType(patientBehaviorSecureBO.getSourceFiveType());
            addDTO.setSourceFiveId(patientBehaviorSecureBO.getSourceFiveId());
            addDTO.setOperatedType(i.getOperatedType());
            addDTO.setOperatedDuration(i.getOperatedDuration());
            addDTO.setIp(i.getIp());
            addDTO.setUa(i.getUa());

            addDTO.setOperatedContent(i.getOperatedContent());
            
            list.add(addDTO);
        });
        patientBehaviorBizService.addBatch(list);
    }

    @Override
    public void simpleSaveBatch(List<PatientBehaviorSaveClientDTO> params) {
        List<PatientBehaviorInfo> list = new ArrayList<>();
        params.forEach(i -> {
            PatientBehaviorInfo entity = new PatientBehaviorInfo();
            entity.setId(i.getId());
            entity.setPatientId(i.getPatientId());
            entity.setUserId(i.getUserId());
            entity.setContentType(i.getContentType());
            entity.setContentId(i.getContentId());
            entity.setSourceOneType(i.getSourceOneType());
            entity.setSourceOneId(i.getSourceOneId());
            entity.setSourceTwoType(i.getSourceTwoType());
            entity.setSourceTwoId(i.getSourceTwoId());
            entity.setSourceThreeType(i.getSourceThreeType());
            entity.setSourceThreeId(i.getSourceThreeId());
            entity.setSourceFourType(i.getSourceFourType());
            entity.setSourceFourId(i.getSourceFourId());
            entity.setSourceFiveType(i.getSourceFiveType());
            entity.setSourceFiveId(i.getSourceFiveId());
            entity.setOperatedType(i.getOperatedType());
            entity.setOperatedDuration(i.getOperatedDuration());
            entity.setIp(i.getIp());
            entity.setUa(i.getUa());

            entity.setOperatedContent(i.getOperatedContent());

            entity.setOperatedStartDate(i.getCreateDate());
            entity.setCreateDate(i.getCreateDate());
            
            list.add(entity);
        });
        patientBehaviorInfoService.saveBatch(list);
    }

}
